% GAME OF LIFE in CPU
% Generate a Grid; in every consecutive generation, look at the number N of "live" neighboors of each cell
% if N = 3, cell turns alive in next step
% if N = 2, cell stays alive in next step
% else, cell dies

function GameOfLife_CPU()

% Measure avarage time?
measure = 0;

% Initialize Grid
gridSize = 500;
numGenerations = 100;
initialGrid = (rand(gridSize,gridSize) > .75);

% Draw the initial grid
hold off
imagesc(initialGrid);
colormap([1 1 1;0 0.5 0]);
title('Initial Grid');

grid = initialGrid;

if measure == 1
    
    timer = tic();
    for generation = 1:numGenerations
        grid = updateGrid(grid, gridSize);
    end
    cpuTime = toc(timer);
    
    imagesc(grid);
    title(num2str(generation));
    drawnow; 
    
    fprintf('Average time on the CPU: %2.3fms per generation.\n', ...
    1000*cpuTime/numGenerations);

elseif measure == 0
    
    % Loop through each generation updating the grid and displaying it
    for generation = 1:numGenerations
        grid = updateGrid(grid, gridSize);

        imagesc(grid);
        title(num2str(generation));
        drawnow;
    end
    
end

%%%%%%>>>>> Evolve grid acording to Game of Life

    function X = updateGrid(X, N)
        p = [1 1:N-1];
        q = [2:N N];
        % Count how many of the eight neighbors are alive.
        neighbors = X(:,p) + X(:,q) + X(p,:) + X(q,:) + ...
            X(p,p) + X(q,q) + X(p,q) + X(q,p);
        % A live cell with two live neighbors, or any cell with
        % three live neighbors, is alive at the next step.
        X = (X & (neighbors == 2)) | (neighbors == 3);
    end


end